import os
import fasttext
from spacy.tokens import Doc
from spacy import util


class LanguageDetector(object):
    def __init__(
        self,
        default_language="xx",
        supported_languages=None,
        threshold=0,
        model_path=os.path.realpath(os.path.join(__file__, "..", "lid.176.ftz")),
    ):
        Doc.set_extension("language", default=default_language, force=True)
        Doc.set_extension("language_score", default=0, force=True)

        self.model = fasttext.load_model(model_path)
        self.default_language = default_language
        self.supported_languages = supported_languages
        self.threshold = threshold

    def __call__(self, doc: Doc):
        labels, confidences = self.model.predict(doc.text)
        label = labels[0]
        confidence = confidences[0]

        language, language_score = self._extract_language(label, confidence)

        doc._.language = language
        doc._.language_score = language_score

        return doc

    def pipe(self, stream, batch_size=128):
        for docs in util.minibatch(stream, size=batch_size):
            labels, confidences = self.model.predict([doc.text for doc in docs])
            for doc, label, confidence in zip(docs, labels, confidences):
                language, language_score = self._extract_language(
                    label[0], confidence[0]
                )
                doc._.language = language
                doc._.language_score = language_score

                yield doc

    def _extract_language(self, label: str, confidence: float):
        language = self.default_language
        if confidence > self.threshold:
            language = label[9:]  # label looks like __label__<ISO code of language>

        if (
            self.supported_languages is not None
            and language not in self.supported_languages
        ):
            language = self.default_language

        return (language, confidence)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['spacy_fastlang']

package_data = \
{'': ['*']}

install_requires = \
['fasttext>=0.9.2,<0.10.0', 'spacy>=2.3.0,<3.0.0']

setup_kwargs = {
    'name': 'spacy-fastlang',
    'version': '0.4.2',
    'description': 'Language detection using FastText and Spacy',
    'long_description': '# spacy_fastlang\n\n## Install\n\nAssuming you have a working python environment, you can simply install it using\n\n```\npip install spacy_fastlang\n```\n\n## Usage\n\nThe library exports a pipeline component called `LanguageDetector` that will set two spacy extensions\n\n- doc.\\_.language = ISO code of the detected language or `xx` as a fallback\n- doc.\\_.language_score = confidence\n\n```\nfrom spacy_fastlang import LanguageDetector\nnlp = spacy.load("...")\nnlp.add_pipe(LanguageDetector())\ndoc = nlp(en_text)\n\ndoc._.language == "..."\ndoc._.language_score >= ...\n```\n\n## Options\n\n[Check the tests](./tests/test_spacy_fastlang.py) to see more examples and available options\n\n## License\n\nEverythin is under `MIT` except the default model which is distributed under [Creative Commons Attribution-Share-Alike License 3.0](https://creativecommons.org/licenses/by-sa/3.0/) by facebook [here](https://fasttext.cc/docs/en/language-identification.html)\n',
    'author': 'Thomas Thiebaud',
    'author_email': 'thiebaud.tom@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/thomasthiebaud/spacy-fastlang',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
